@@ -24,9 +24,6 @@
 #endif
 
 #include <config.h>
-#if HAVE_INTTYPES_H
-# include <inttypes.h>
-#endif
 #include <stdio.h>
 #include <sys/types.h>
 #include <getopt.h>
@@ -46,7 +43,7 @@
 #define PROGRAM_NAME "df"
 
 #define AUTHORS \
-  "Torbjorn Granlund, David MacKenzie, Larry McVoy, and Paul Eggert"
+  N_ ("Torbjorn Granlund, David MacKenzie, Larry McVoy, and Paul Eggert")
 
 char *xgetcwd ();
 
